<?php
/**
 * Plugin Name: Elementor PDF Viewer for WordPress Addon
 * Description: This addon helps to create PDF Viewer for WordPress shortcodes using Elementor interface
 * Plugin URI:  https://themencode.com/elementor-addon-pdf-viewer-for-wordpress/
 * Version:     1.1.0
 * Author:      ThemeNcode
 * Author URI:  https://themencode.com/
 * Text Domain: elementor-pvfw
 *
 * @package  elementor-pvfw
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Main Elementor PDF Viewer for WordPress Addon Class
 *
 * The main class that initiates and runs the plugin.
 *
 * @since 1.0.0
 */
final class Elementor_PVFW_Addon {

	/**
	 * Plugin Version
	 *
	 * @since 1.0.0
	 *
	 * @var string The plugin version.
	 */
	const VERSION = '1.1.0';

	/**
	 * Minimum Elementor Version
	 *
	 * @since 1.0.0
	 *
	 * @var string Minimum Elementor version required to run the plugin.
	 */
	const MINIMUM_ELEMENTOR_VERSION = '2.0.0';

	/**
	 * Minimum PHP Version
	 *
	 * @since 1.0.0
	 *
	 * @var string Minimum PHP version required to run the plugin.
	 */
	const MINIMUM_PHP_VERSION = '7.0';

	/**
	 * Instance
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @static
	 *
	 * @var Elementor_PVFW_Addon The single instance of the class.
	 */
	private static $_instance = null;

	/**
	 * Instance
	 *
	 * Ensures only one instance of the class is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @static
	 *
	 * @return Elementor_PVFW_Addon An instance of the class.
	 */
	public static function instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;

	}

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function __construct() {

		add_action( 'init', array( $this, 'i18n' ) );
		add_action( 'plugins_loaded', array( $this, 'init' ) );

	}

	/**
	 * Load Textdomain
	 *
	 * Load plugin localization files.
	 *
	 * Fired by `init` action hook.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function i18n() {

		load_plugin_textdomain( 'elementor-pvfw' );

	}

	/**
	 * Initialize the plugin
	 *
	 * Load the plugin only after Elementor (and other plugins) are loaded.
	 * Checks for basic plugin requirements, if one check fail don't continue,
	 * if all check have passed load the files required to run the plugin.
	 *
	 * Fired by `plugins_loaded` action hook.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function init() {

		/**
		 * Check if plugin is active
		 *
		 * @param  [type] $plugin [description].
		 *
		 * @return [type]         [description]
		 */
		function pvfw_is_plugin_active( $plugin ) {
			return in_array( $plugin, (array) get_option( 'active_plugins', array() ) );
		}

		// Check if PDF Viewer for WordPress installed and activated
		if ( !pvfw_is_plugin_active('pdf-viewer-for-wordpress/pdf-viewer-for-wordpress.php') ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_missing_pvfw_plugin' ] );
			return;
		}

		// Check if Elementor installed and activated
		if ( ! did_action( 'elementor/loaded' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_missing_main_plugin' ] );
			return;
		}

		// Check for required Elementor version
		if ( ! version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_minimum_elementor_version' ] );
			return;
		}

		// Check for required PHP version
		if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_minimum_php_version' ] );
			return;
		}

		// Add Plugin actions
		add_action( 'elementor/widgets/widgets_registered', [ $this, 'init_widgets' ] );
		add_action( 'elementor/controls/controls_registered', [ $this, 'init_controls' ] );

	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have Elementor installed or activated.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_missing_main_plugin() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor */
			esc_html__( '"%1$s" requires "%2$s" to be installed and activated.', 'elementor-pvfw' ),
			'<strong>' . esc_html__( 'Elementor PDF Viewer for WordPress Addon', 'elementor-pvfw' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'elementor-pvfw' ) . '</strong>'
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have PDF Viewer for WordPress installed or activated.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_missing_pvfw_plugin() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: PDF Viewer for WordPress */
			esc_html__( '"%1$s" requires "%2$s" to be installed and activated.', 'elementor-pvfw' ),
			'<strong>' . esc_html__( 'Elementor PDF Viewer for WordPress Addon', 'elementor-pvfw' ) . '</strong>',
			'<strong>' . esc_html__( 'PDF Viewer for WordPress', 'elementor-pvfw' ) . '</strong>'
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required Elementor version.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_minimum_elementor_version() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor 3: Required Elementor version */
			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'elementor-pvfw' ),
			'<strong>' . esc_html__( 'Elementor PDF Viewer for WordPress Addon', 'elementor-pvfw' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'elementor-pvfw' ) . '</strong>',
			 self::MINIMUM_ELEMENTOR_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required PHP version.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_minimum_php_version() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'elementor-pvfw' ),
			'<strong>' . esc_html__( 'Elementor PDF Viewer for WordPress Addon', 'elementor-pvfw' ) . '</strong>',
			'<strong>' . esc_html__( 'PHP', 'elementor-pvfw' ) . '</strong>',
			 self::MINIMUM_PHP_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	/**
	 * Init Widgets
	 *
	 * Include widgets files and register them
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function init_widgets() {

		// Include Widget files
		require_once( __DIR__ . '/widgets/link-widget.php' );
		require_once( __DIR__ . '/widgets/iframe-widget.php' );
		require_once( __DIR__ . '/widgets/image-widget.php' );
		require_once( __DIR__ . '/widgets/navigative-widget.php' );

		// Register widget
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \Elementor_PVFW_Link_SC_Widget() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \Elementor_PVFW_iFrame_SC_Widget() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \Elementor_PVFW_Image_SC_Widget() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \Elementor_PVFW_Nav_SC_Widget() );
	}

	/**
	 * Init Controls
	 *
	 * Include control files and register them
	 *
	 * @since 1.1.0
	 *
	 * @access public
	 */
	public function init_controls() {

		// Include Widget files
		require_once( __DIR__ . '/controls/fileselect-control.php' );

		// Register controls
		\Elementor\Plugin::$instance->controls_manager->register_control( 'file-select', new \FileSelect_Control() );
	}

}

Elementor_PVFW_Addon::instance();